import java.awt.Graphics2D;
import java.awt.Point;
import java.util.ArrayList;

/**
 * This class manages the 8 play cards columns 
 * 
 * @author figgy
 *
 */
public class CardCascadeArea implements CardPlayableArea{

	private static final int MAX_CARD_COLUMNS = 8;
	private static final int NO_COLUMN = -1;
	
	private static int START_LEFT_X = 10;
	private static final int START_LEFT_Y = 160;
	
	private CardTableau[] cardColumns;
	
	private CardOpenArea cardOpenArea;			// Holds the temporary card area (top left)
	
	public CardCascadeArea(CardOpenArea cardTempArea) {
		// save the information on the I donother areas
		this.cardOpenArea = cardTempArea;
		
		START_LEFT_X = (FreeCell.DISPLAY_WIDTH - (MAX_CARD_COLUMNS * CardDeck.CARD_WIDTH) - ((MAX_CARD_COLUMNS-1) * CARD_GAP) ) / 2;
		cardColumns = new CardTableau[MAX_CARD_COLUMNS];
		
		// set up location of each card column list area
		for (int i=0; i < MAX_CARD_COLUMNS; i++) {
			cardColumns[i] = new CardTableau(START_LEFT_X + (i * CardDeck.CARD_WIDTH) 
					+ (i * CARD_GAP), START_LEFT_Y);
		}
	}
	
	private int findColumnPointIsIn(Point point) {
		for (int i=0; i < cardColumns.length; i++) {
			if (cardColumns[i].isPointInColCardSpace(point)) {
				return i;
			}
		}
		return NO_COLUMN;
	}
	
	public int numAvailableSlotsExceptTargetAndSource(int columnTarget, int columnSource) {
		if (cardColumns == null) return 0;
		int numberOfSlotsAvailable = 0;
		for (int i = 0; i < cardColumns.length; i++) {
			if (i == columnTarget || i == columnSource) continue;
			if (!cardColumns[i].hasCards()) {
				++numberOfSlotsAvailable;
			}
		}
		return numberOfSlotsAvailable;
	}
	
	/** 
	 * This method spreads the cards provided to each card column until all 52 are distributed
	 * 
	 * @param deck
	 */
	public void dealTheCards(CardDeck deck) {
		int currentColumn = 0;
		for (int i=0; i < CardDeck.NUM_CARDS_IN_DECK; i++) {
			cardColumns[currentColumn].addCardFreely(deck.getCard(i));
			++currentColumn;
			if (currentColumn >= MAX_CARD_COLUMNS) {
				currentColumn = 0;
			}
		}
	}
	
	public void render(Graphics2D g2D) {
		for (int i=0; i < MAX_CARD_COLUMNS; i++) {
			cardColumns[i].render(g2D);
		}
	}
	
	@Override
	public boolean isPointInArea(Point point) {
		for (int i=0; i < cardColumns.length; i++) {
			if (cardColumns[i].isPointInColCardSpace(point)) {
				return true;
			}
		}
		
		return false;
	}

	@Override
	public boolean isCardHere(Point point) {
		// First find the column this point could be in then get card list (if any)
		for (int i=0; i < MAX_CARD_COLUMNS; i++) {
			if (cardColumns[i].hasCards()) {
				// check that point falls into card space
				if (cardColumns[i].hasCardHere(point)) return true;
			}
		}
		return false;
	}
	
	private int calculateMaxCardsCanMove(int columnTarget, int columnSource) {
		return Math.max(cardOpenArea.numAvailableSlots()+1,
				(cardOpenArea.numAvailableSlots()+1) * (numAvailableSlotsExceptTargetAndSource(columnTarget, columnSource) + 1));
	}
	
	@Override
	public ArrayList<Card> grabCards(Point point, boolean removeFromList) {
		// first find column
		for (int i=0; i < MAX_CARD_COLUMNS; i++) {
			if (cardColumns[i].hasCards()) {
				if (cardColumns[i].hasCardHere(point)) {
					// this is the column....return cardList (automatically removed)
					//int numAvailableCardSlots = calculateMaxCardsCanMove(); 
					ArrayList<Card> cardList = cardColumns[i].getCardListHere(point, 100, removeFromList);
					if (cardList != null && cardList.size() > 0) {
						return cardList;
					} else {
						return null;
					}
				}
			}
		}
		return null;
	}

	// Cards can be dropped in the space if there is enough available space
	private boolean enoughSpaceToMoveCards(int columnTarget, int columnSource, int numCardsBeingDropped) {
		return numCardsBeingDropped <= calculateMaxCardsCanMove(columnTarget, columnSource);
	}
	
	@Override
	public boolean dropCard(ArrayList<Card> dropCards, Point point, boolean validate) {
		if (dropCards == null || dropCards.size() == 0) return false;
		// First find the source column
		int sourceCol = this.findColumnPointIsIn(dropCards.get(0).getScreenPosition());
		// First find the column than add to the end
		for (int i=0; i < MAX_CARD_COLUMNS; i++) {
			if (cardColumns[i].isPointInColCardSpace(point)) {	// this is the column to add the card(s) too.
				// this is the column....
				if (validate && !enoughSpaceToMoveCards(i, sourceCol, dropCards.size())) {
					return false;
				}
				
				for (Card card:dropCards) {
					boolean bResult = cardColumns[i].addCard(card, validate);
					if (!bResult) {
						return false;
					}
				}
				return true;
			} 
		}
		
		return false;
	}

	@Override
	public Card findCardToAutoMove(CardAutoAddable autoAddableArea) {
		for (int i=0; i < MAX_CARD_COLUMNS; i++) {
			Card topCard = cardColumns[i].getTopCard();
			if (topCard != null && autoAddableArea.canCardBeAddedAutomatically(topCard)) {
				return topCard;
			}
		}
		return null;
	}

	@Override
	public int numAvailableSlots() {
		int count = 0;
		for (CardTableau tableau: cardColumns) {
			if (!tableau.hasCards()) {
				++count;
			}
		}
		return count;
	}

	@Override
	public Card getCardHere(Point point) {
		Card card = null;
		for (int i=0; i < MAX_CARD_COLUMNS; i++) {
			card = cardColumns[i].getCardHere(point);
			if (card != null) break;
		}
		return card;
	}
	



}
