import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;

/**
 * This class manages the card suit collection we will maintain starting from 
 * ACE up in order (to KING) There is one for each FACE type
 * 
 * @author figgy
 *
 */
public class CardFoundationArea implements CardPlayableArea, CardAutoAddable {

	private Card[] cards; // This holds the highest card in a suit (that displays)
	
	private static final int MAX_TEMP_CARDS = 4;
	
	private static final int TOP_LEFT_CARD_X = 400;	// starting location of cards in this container
	private static final int TOP_LEFT_CARD_Y = 15;
	
	/*
	 * Constructor initializes 
	 */
	public CardFoundationArea() {
		cards = new Card[MAX_TEMP_CARDS];
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			cards[i] = null;
		}
	}
	
	public boolean allCardsInPlace() {
		for (Card aCard: cards) {
			if (aCard == null || aCard.getFace() != CardDeck.FACE.King) return false;
		}
		return true;
	}
	
	public void render(Graphics2D g2D) {
		g2D.setColor(Color.white);
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			int x = TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP);
			int y = TOP_LEFT_CARD_Y;
			
			if (cards[i] == null) {
				// draw rounded rectangle
				g2D.drawRoundRect(x, y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT, CARD_ROUND_ARC, CARD_ROUND_ARC);
			} else {
				// draw card image
				cards[i].render(g2D);
				//g2D.drawImage(cards[i].getImage(),null, x, y);
			}
		}
	}
	
	@Override
	public boolean isPointInArea(Point point) {
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			int x = TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP);
			int y = TOP_LEFT_CARD_Y;
			
			if (CollisionDetection.pointInRect(new Rectangle(x, y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT), point)) {
				return true;
			}
			
		}
		return false;	
	}

	@Override
	public boolean isCardHere(Point point) {
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			int x = TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP);
			int y = TOP_LEFT_CARD_Y;
			
			if (cards[i] != null) {
				// check of point resides in the space for this card
				if (CollisionDetection.pointInRect(new Rectangle(x, y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT), point)) {
					return true;
				}
			}
			
		}
		return false;
	}
	public Point getCardPointHere(Point point) {
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			int x = TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP);
			int y = TOP_LEFT_CARD_Y;
			
			if (CollisionDetection.pointInRect(new Rectangle(x, y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT), point)) {
					return new Point(x, y);
			}
		}
		
		return null;
		
	}
	public int getCardIndexHere(Point point) {
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			int x = TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP);
			int y = TOP_LEFT_CARD_Y;
			
			if (CollisionDetection.pointInRect(new Rectangle(x, y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT), point)) {
					return i;
			}
		}
		
		return -1;
		
	}
	@Override
	public ArrayList<Card> grabCards(Point point, boolean removeFromList) {
		// We don't move cards from this area
		return null;
	}

	@Override
	public boolean dropCard(ArrayList<Card> dropCards, Point point, boolean validate) {
		// We only handle ONE CARD AT A TIME
		if (dropCards == null || dropCards.size() != 1) return false;
		Card card = dropCards.get(0);
		
		// Obtain index of location
		int cardIndex = getCardIndexHere(point);
		if (!validate) {
			cards[cardIndex] = card;	// just returning the card back
			return true;
		}
		
		Point slotTopLeftPoint = getCardPointHere(point);
		if (cards[cardIndex] == null) {
			// just add the card if it is an Ace
			if (card.getFace() == CardDeck.FACE.Ace) {
				card.setScreenPosition(slotTopLeftPoint);
				cards[cardIndex] = card;
				return true;
			} else {
				return false;
			}
		} else {
			// check if we can add here (same suit and exactly the next card!
			Card topCard = cards[cardIndex];
			// Check suit matches
			if (!suitMatches(card,topCard)) return false;
			
			// Make a special case for Ace 
			if (topCard.getFace() == CardDeck.FACE.Ace &&
					card.getFace() == CardDeck.FACE.Two ) {
				// allow adding this card
				card.setScreenPosition(slotTopLeftPoint);
				cards[cardIndex] = card;
				return true;
			} else {
				// 2, 3, ...10, J, Q, K
				if ( (card.getFace().ordinal()+1) == topCard.getFace().ordinal()) {
					card.setScreenPosition(slotTopLeftPoint);
					cards[cardIndex] = card;
					return true;
				}
			}
		}
		return false;
	}	
	
	/**
	 * 	This method finds the SUIT foundation the card belongs in and sends back the top left coordinate
	 * 	in order to be used for automatic animation of the card into the foundation slot
	 * @param card
	 * @return
	 */
	public Point findCardPoint(Card card) {
		// returns the point position from the slot this card would fit in
		Point newCardPoint = null;
		if (card.getFace() == CardDeck.FACE.Ace) {
			// find first available spot
			for (int i=0; i < cards.length; i++) {
				if (cards[i] == null) {
					newCardPoint = new Point(TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP), TOP_LEFT_CARD_Y);
					break;
				}
			}
		} else {
			// find the foundation slot that already is collecting this SUIT
			for (int i=0; i < cards.length; i++) {
				if (cards[i] != null && suitMatches(cards[i],card) 
						&& canCardBePlaced(cards[i], card) ) {
					newCardPoint = new Point(TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP), TOP_LEFT_CARD_Y);
					break;
				}
			}
		}
		
		return newCardPoint;
	}
	
	private boolean suitMatches(Card foundationCard, Card candidateCard) {
		if (foundationCard == null || candidateCard == null) return false;
		return foundationCard.getSuit() == candidateCard.getSuit();
	}
	
	// This checks that the card being considered to automatically add to foundation is
	// is not too far ahead of cards of opposite color. For example 4S (4 of spades) can
	// not be automatically added unless 3H (3 of clubs) and 3D are already on the foundation
	private boolean otherColorsInSuit(Card card) {
		int colorCnt = 0;	// looking for two cards of opposite color less than card
		if (cards == null) return false;
		for (int i=0; i < cards.length; i++) {
			Card foundationCard = cards[i];
			if (foundationCard != null) {
				int foundationCardOrdinal = (foundationCard.getFace() == CardDeck.FACE.Ace 
						? CardDeck.FACE.Two.ordinal()+1 : foundationCard.getFace().ordinal());
				if ( !suitMatches(foundationCard, card)) {
					if ((CardDeck.cardColor[foundationCard.getSuit().ordinal()] != CardDeck.cardColor[card.getSuit().ordinal()] &&
							(foundationCardOrdinal <= card.getFace().ordinal()+1))) {
						++colorCnt;
					}
				}
			}
		}	
		return (colorCnt == 2);
	}
	
	private boolean canCardBePlaced(Card foundationCard, Card candidateCard) {		
		if (!suitMatches(foundationCard,candidateCard)) return false;
		if ((foundationCard.getFace() == CardDeck.FACE.Ace && candidateCard.getFace() == CardDeck.FACE.Two))
			return true;
		return ((foundationCard.getFace().ordinal() == candidateCard.getFace().ordinal()+1) &&
					otherColorsInSuit(candidateCard));
	}
	
	
	@Override
	public boolean canCardBeAddedAutomatically(Card card) {
		// do the easy checks first
		if (card.getFace() == CardDeck.FACE.Ace) return true;
		
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			if (cards[i] != null) {
				if (suitMatches(cards[i],card)) {
					return canCardBePlaced(cards[i], card); 
				}
			}
		}
	
		return false;
	}



	@Override
	public int numAvailableSlots() {
		if (cards == null || cards.length == 0) return 0;
		int cntOfAvailable = 0;
		for (int i=0; i < cards.length; i++) {
			if (cards[i] == null) {
				++cntOfAvailable;
			}
		}
		return cntOfAvailable;
	}

	@Override
	public Card findCardToAutoMove(CardAutoAddable autoAddableArea) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Card getCardHere(Point point) {
		// TODO Auto-generated method stub
		return null;
	}
}
