import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;

/**
 * This class defines and manages the object in the game for the temporary card area
 * 
 * For this specific game it is set at the top left-hand side of the game. It will 
 * allow the user to drag a drop four cards into this temporary card area
 * 
 * @author figgy
 *
 */
public class CardOpenArea implements CardPlayableArea {
	private static final int MAX_TEMP_CARDS = 4;
	
	private static final int TOP_LEFT_CARD_X = 10;	// starting location of cards in this container
	private static final int TOP_LEFT_CARD_Y = 15;
	
	private static final int CARD_GAP = 10;			// space between cards
	private static final int CARD_ROUND_ARC = 10;
	
	private Card cards[];
	
	public CardOpenArea() {
		cards = new Card[MAX_TEMP_CARDS]; // allocate enough space to hold MAX_TEMP_CARDS
		
		// Game starts with nothing in this area
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			cards[i] = null;
		}
	}
	
	public int numAvailableSlots() {
		if (cards == null) return 0;
		int numberOfEmtptySlots = 0;
		for (Card aCard : cards) {
			if (aCard == null) {
				++numberOfEmtptySlots;
			}
		}
		return numberOfEmtptySlots;
	}
	
	
	/**
	 * 	finds the top-left Point position of the first temp slot found
	 * 
	 * @return
	 */
	public Point findAvailablePoint() {
		Point newCardPoint = null;
		for (int i=0; i < cards.length; i++) {
			if (cards[i] == null) {
				newCardPoint = new Point(TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP),TOP_LEFT_CARD_Y);
				break;
			}
		}
		return newCardPoint;
	}
	/**
	 * returns the slot position if the Point(x,y) corresponds to an available temp slot
	 * otherwise returns -1
	 * @param x
	 * @param y
	 * @return
	 */
	public int canDropCardHere(int x, int y) {
		int slotPosition = -1;
		int maxXPosition = TOP_LEFT_CARD_X + (MAX_TEMP_CARDS * CardDeck.CARD_WIDTH) + (MAX_TEMP_CARDS-1) * CARD_GAP;
		if (x < TOP_LEFT_CARD_X || x > maxXPosition || 
				y < TOP_LEFT_CARD_Y || y > TOP_LEFT_CARD_Y + CardDeck.CARD_HEIGHT) {
			return slotPosition;
		}
		
		// check each card position
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			if (cards[i] != null) continue;
			int rectX = TOP_LEFT_CARD_X + i * CardDeck.CARD_WIDTH + i * CARD_GAP;
			Rectangle rect = new Rectangle(rectX, TOP_LEFT_CARD_Y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT);
			if (CollisionDetection.pointInRect(rect, new Point(x, y))) {
				slotPosition = i;
				break;	// send this slotPosition back
			}
		}
		
		return slotPosition;
	}
	
	public void dropCardInSlot(int slot, Card card) {
		if (slot < 0 || slot >= MAX_TEMP_CARDS) return;
		if (cards[slot] == null) {
			cards[slot] = card;
		} else {
			System.out.println("Slot already occupied!");
		}
	}
	/**
	 * Draw the cards in the temporary area
	 * @param g2D
	 */
	public void render(Graphics2D g2D) {
		g2D.setColor(Color.white);
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			int x = TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP);
			int y = TOP_LEFT_CARD_Y;
			
			if (cards[i] == null) {
				// draw rounded rectangle
				g2D.drawRoundRect(x, y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT, CARD_ROUND_ARC, CARD_ROUND_ARC);
			} else {
				// draw card image
				//g2D.drawImage(cards[i].getImage(),null, x, y);
				cards[i].render(g2D);
			}
		}
	}
	@Override
	public boolean isPointInArea(Point point) {
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			int x = TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP);
			int y = TOP_LEFT_CARD_Y;
			Rectangle rect = new Rectangle(x, y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT);
			if (CollisionDetection.pointInRect(rect, point)) {
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean isCardHere(Point p) {
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			int x = TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP);
			int y = TOP_LEFT_CARD_Y;
			Rectangle rect = new Rectangle(x, y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT);
			if (CollisionDetection.pointInRect(rect, p)) {
				if (cards[i] != null) return true;
			}
		}
		return false;
	}

	@Override
	public ArrayList<Card> grabCards(Point point, boolean removeFromList) {
		ArrayList<Card> cardList = null;
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			int x = TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP);
			int y = TOP_LEFT_CARD_Y;
			Rectangle rect = new Rectangle(x, y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT);
			if (CollisionDetection.pointInRect(rect, point)) {
				if (cards[i] != null) {
					cardList = new ArrayList<Card>();
					cardList.add(cards[i]);
					if (removeFromList) {
						cards[i] = null;
					}
				}
			}
		}
		return cardList;
	}

	@Override
	public boolean dropCard(ArrayList<Card> dropCards, Point point, boolean validate) {
		// Check that this is only one card
		if (dropCards == null || dropCards.size() != 1) return false;
		Card card = dropCards.get(0);
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			int x = TOP_LEFT_CARD_X + (i * CardDeck.CARD_WIDTH) + (i * CARD_GAP);
			int y = TOP_LEFT_CARD_Y;
			Rectangle rect = new Rectangle(x, y, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT);
			if (CollisionDetection.pointInRect(rect, point) && cards[i] == null) {
				card.setScreenPosition(new Point(x,y));
				cards[i] = card;
				return true;
			}
		}
		return false;
	}

	@Override
	public Card findCardToAutoMove(CardAutoAddable autoAddableArea) {
		for (int i=0; i < MAX_TEMP_CARDS; i++) {
			Card card = cards[i];
			if (card != null) {
				if (autoAddableArea.canCardBeAddedAutomatically(card)) {
					return card;
				}
			}
		}
		return null;
	}

	@Override
	public Card getCardHere(Point point) {
		// TODO Auto-generated method stub
		return null;
	}


	
}
