import java.awt.Graphics2D;
import java.awt.Point;
import java.util.ArrayList;

/**
 * This class handles all the card playfields:
 * 
 * - Playable columns where all the cards are initially placed
 * - Temp card area
 * - Suit collection area
 * 
 * @author figgy
 *
 */
public class CardPlayField {
	
	public enum CARD_AREA  {tempArea, suitListArea, playArea, noArea }
	
	private CardOpenArea cardOpenArea;				// Holds the temporary card area (top left)
	private CardFoundationArea cardFoundationArea;	// Holds the build suit list area (top right)
	private CardCascadeArea cardPlayArea;				// Holds the area containing all the cards intially dealt
	
	/**
	 * This constructor creates all three play fields and deals the cards to the cardPlayArea
	 * @param deck
	 */
	public CardPlayField(CardDeck deck) {
		cardOpenArea = new CardOpenArea();
		cardFoundationArea = new CardFoundationArea();
		cardPlayArea = new CardCascadeArea(cardOpenArea);
		cardPlayArea.dealTheCards(deck);
	}
	
	public boolean isCardUnderPoint(Point p) {
		return cardOpenArea.isCardHere(p) || cardFoundationArea.isCardHere(p)
					|| cardPlayArea.isCardHere(p);
	}
	
	/**
	 * This method checks that user clicked on card that is indeed "grabable"
	 * 
	 * @param p
	 * @return
	 */
	public boolean isMovableCardUnderPoint(Point p) {
		CARD_AREA thisArea = whichAreaIsPoint(p);
		if (thisArea == CARD_AREA.suitListArea) return false;	// can't move from here
		return cardOpenArea.isCardHere(p) || cardPlayArea.isCardHere(p);
	}
	
	public CARD_AREA whichAreaIsPoint(Point p) {
		if (cardOpenArea.isPointInArea(p)) return CARD_AREA.tempArea;
		if (cardFoundationArea.isPointInArea(p)) return CARD_AREA.suitListArea;
		if (cardPlayArea.isPointInArea(p)) return CARD_AREA.playArea;
		return CARD_AREA.noArea;
	}

	public Card getValidPeakCard(Point p) {
		Card peakCard = null;
		CARD_AREA thisArea = whichAreaIsPoint(p);
		if (!thisArea.equals(CARD_AREA.playArea)) return peakCard;
		peakCard = cardPlayArea.getCardHere(p);
		return peakCard;
	}
	
	public boolean isSingleClickInValidArea(Point p) {
		// double click is only valid in play area
		CARD_AREA thisArea = whichAreaIsPoint(p);
		if (thisArea.equals(CARD_AREA.playArea) || thisArea.equals(CARD_AREA.tempArea)) return true;
		return false;
	}
	
	public boolean isDoubleClickInValidArea(Point p) {
		// double click is only valid in play area
		CARD_AREA thisArea = whichAreaIsPoint(p);
		if (thisArea.equals(CARD_AREA.playArea)) return true;
		return false;
	}
	
	public ArrayList<Card> pickUpCards(Point p, boolean removeFromTarget) {
		// find area
		CARD_AREA thisArea = whichAreaIsPoint(p);
		switch (thisArea) {
			case tempArea:
				return cardOpenArea.grabCards(p, removeFromTarget);
			case suitListArea:
				return cardFoundationArea.grabCards(p, removeFromTarget);
			case playArea:
				return cardPlayArea.grabCards(p, removeFromTarget);
			case noArea:
				return null;
		}
		return null;
	}
	
	public boolean dropCard(ArrayList<Card> cardList, Point dropPoint) {
		// find area
		CARD_AREA thisArea = whichAreaIsPoint(dropPoint);
		if (cardList == null || cardList.size() == 0) return false;
		switch (thisArea) {
			case tempArea:
				return cardOpenArea.dropCard(cardList, dropPoint, true);
			case suitListArea:
				return cardFoundationArea.dropCard(cardList, dropPoint, true);
			case playArea:
					return cardPlayArea.dropCard(cardList, dropPoint, true);
			case noArea:
				return false;
		}
		return false;
	}
	
	/**
	 * This method finds the location (point) in either tempArea or suitArea that this
	 * card will move to and send it back (if no adequate spot found then it returns
	 * null. 
	 * 
	 * @param cardList expects only 1 card in cardList
	 * @return
	 */
	public Point determineCardMoveArea(ArrayList<Card>cardList) {
		Point newCardPoint = null;
		
		if (cardList == null || cardList.size() == 0) return newCardPoint;
		
		// Get the one card that should be in the card list
		Card card = cardList.get(0);
		if (cardFoundationArea.canCardBeAddedAutomatically(card)) {
			newCardPoint = cardFoundationArea.findCardPoint(card);
		} else {
			if (cardOpenArea.numAvailableSlots() > 0) {
				newCardPoint = cardOpenArea.findAvailablePoint();
			}
		}
		return newCardPoint;
	}

	/**
	 * This method is used to directly return a set of cards back to its originating spot
	 * @param cardList
	 * @param point
	 */
	public void returnCards(ArrayList<Card>cardList, Point point) {
		if (cardList == null || cardList.size() == 0) return;
		
		// Find the area these cards belong to
		CARD_AREA thisArea = whichAreaIsPoint(point);
		switch (thisArea) {
			case tempArea:
				cardOpenArea.dropCard(cardList, point, false);
				break;
			case suitListArea:
				cardFoundationArea.dropCard(cardList, point, false);
				break;
			case playArea:
				cardPlayArea.dropCard(cardList, point, false);
				break;
			case noArea:
				break;
		}
	}

	public boolean isGameOver() {
		return cardFoundationArea.allCardsInPlace();
	}
	
	/** This function is called whenever user is not in the middle of any action.
	 * 	It will iterate through all the cards in the temp area and play area (until known found)
	 *  and move card that can automatically go to foundation suit area.
	 *  
	 */
	public Card autoMoveAnyCards() {
		Card card = cardOpenArea.findCardToAutoMove(cardFoundationArea);
		if (card == null) {
			card = cardPlayArea.findCardToAutoMove(cardFoundationArea);
		} 
		return card;
	}
	

	
	public void render(Graphics2D g2D) {	
		cardOpenArea.render(g2D);
		cardFoundationArea.render(g2D);
		cardPlayArea.render(g2D);
	}
}
