import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;

/**
 * This class manages a card column.
 * It manages the list of cards on the list and displays the list when asked to draw
 * 
 * @author figgy
 *
 */
public class CardTableau {
	private static final int CARD_GAP_Y = 20;
	private static final int CARD_ROUND_ARC = 10;
	
	private int topLeftX;
	private int topLeftY;
	
	ArrayList<Card> cards;
	
	public CardTableau(int topLeftX, int topLeftY) {
		this.topLeftX = topLeftX;
		this.topLeftY = topLeftY;
		
		cards = new ArrayList<Card>();

	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		if (cards == null || cards.size() == 0) return "";
		for (Card aCard : cards) {
			sb.append(aCard.toString() + "\n");
		}
		return sb.toString();
	}
	
	public boolean hasCards() {
		return (cards != null && cards.size() > 0);
	}
	
	public int getTopLeftX() {
		return topLeftX;
	}

	public void setTopLeftX(int topLeftX) {
		this.topLeftX = topLeftX;
	}

	public int getTopLeftY() {
		return topLeftY;
	}

	public void setTopLeftY(int topLeftY) {
		this.topLeftY = topLeftY;
	}

	/**
	 * All the cards in card list must be in decreasing order
	 * @param cardList
	 * @return
	 */
	public boolean isCardListValid(ArrayList<Card> cardList) {
		if (cardList == null || cardList.size() == 0) return false;
		
		if (cardList.size() == 1) return true;	// only one card in list
		
		for (int i=0; i < cardList.size()-1; i++) {
			if (cardList.get(i).getFace().ordinal() != cardList.get(i+1).getFace().ordinal()+1) {
				return false;
			}
		}
		return true;
	}
	
	private boolean isCardMoveValid(ArrayList<Card> cardList, int numberOfAvailableSlots) {
		if (cardList == null || cardList.size() <= 1)  {
			return true;
		}
		if (cardList.size() > numberOfAvailableSlots) return false;
		for (int i=0; i < cardList.size()-1; i++) {
			Card aboveCard = cardList.get(i);
			Card belowCard = cardList.get(i+1);
			if (aboveCard.getFace().ordinal()+1 != belowCard.getFace().ordinal()) return false;
			if ( CardDeck.cardColor[aboveCard.getSuit().ordinal()] == 
					CardDeck.cardColor[belowCard.getSuit().ordinal()] ) return false;
		}
		return true;
	}
	/**
	 * This returns the list of cards at p and below iff they are decreasing in size and 
	 * alternate in color
	 * @param x
	 * @param y
	 * @return
	 */ 
	/*
	public ArrayList<Card> getCardListHere(Point p, int numberOfAvailableSlots) {
		return getCardListHere(p, numberOfAvailableSlots, true);
	}	
	*/
	
	// This should allow cards to be removed without consideration of the number of slota
	public ArrayList<Card> getCardListHere(Point p, int numberOfAvailableSlots, boolean removeCards) {
		ArrayList<Card> cardList = null;
		// if there are no cards in this column OR the mouse is too far left or right OR above
		// then don't bother checking
		if (cards.size() == 0 || p.getX() < topLeftX 
				|| p.getX() > topLeftX+CardDeck.CARD_WIDTH || p.getY() < topLeftY) return cardList;
		
		// determine Rectangular area for the highest or last card
		int lastCardX = topLeftX;
		for (int i=0; i < cards.size(); i++) {
			int lastCardY = topLeftY + (i*CARD_GAP_Y);
		
			Rectangle rect = new Rectangle(lastCardX, lastCardY, CardDeck.CARD_WIDTH,  
					(i==cards.size()-1 ? CardDeck.CARD_HEIGHT: CARD_GAP_Y) );
			if (CollisionDetection.pointInRect(rect, p)) {				
				// this mouse position overlaps this card...send it and all cards below it (if valid)
				cardList = new ArrayList<Card>();
				for (int j=i; j < cards.size(); j++) {
					Card newCard = cards.get(j);
					cardList.add(newCard);
					if (!isCardMoveValid(cardList, numberOfAvailableSlots)) {
						return null;
					}
				}
				// remove cards from i..end
				if (removeCards) {
					cards.removeAll(cardList);
				}
				return cardList;
			}
		}
		return cardList;		
	}

	
	
	/**
	 * This method returns true if the point(x,y) location on screen corresponds to this card
	 * column space
	 * @param point
	 * @return
	 */
	public boolean isPointInColCardSpace(Point point) {
		Rectangle rect;
		// if empty then send back true is point falls into first card space
		if (cards == null || cards.size() == 0) {
			rect = new Rectangle(topLeftX, topLeftY, 
					CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT);
		} else {
			rect = new Rectangle(topLeftX, topLeftY, CardDeck.CARD_WIDTH, 
							CardDeck.CARD_HEIGHT + CARD_GAP_Y * cards.size()-1);
		}
		return CollisionDetection.pointInRect(rect, point);
			
	}
	
	public boolean hasCardHere(Point p) {	
		// if there are no cards in this column OR the mouse is too far left or right OR above
		// then don't bother checking
		if (cards.size() == 0 || p.getX() < topLeftX 
				|| p.getX() > topLeftX+CardDeck.CARD_WIDTH || p.getY() < topLeftY) return false;
		
		
		// determine Rectangular area for the highest or last card
		int lastCardX = topLeftX;
		for (int i=0; i < cards.size(); i++) {
			int lastCardY = topLeftY + (i*CARD_GAP_Y);
		
			Rectangle rect = new Rectangle(lastCardX, lastCardY, CardDeck.CARD_WIDTH, 
					(i==cards.size()-1 ? CardDeck.CARD_HEIGHT: CARD_GAP_Y));
		
			if (CollisionDetection.pointInRect(rect, p)) {
				return true;
			}
		}
		return false;
	}		
	
	public Card getCardHere(Point point) {
		Card card = null;
		
		if (cards.size() == 0 || point.getX() < topLeftX 
				|| point.getX() > topLeftX+CardDeck.CARD_WIDTH || point.getY() < topLeftY) return card;
		
		
		// determine Rectangular area for the highest or last card
		int lastCardX = topLeftX;
		for (int i=0; i < cards.size(); i++) {
			int lastCardY = topLeftY + (i*CARD_GAP_Y);
		
			Rectangle rect = new Rectangle(lastCardX, lastCardY, CardDeck.CARD_WIDTH, 
					(i==cards.size()-1 ? CardDeck.CARD_HEIGHT: CARD_GAP_Y));
		
			if (CollisionDetection.pointInRect(rect, point)) {
				card = cards.get(i);
				break;
			}
		}
		return card;
		
	}
	
	public Card removeTopCard() {
		Card topCard = null;
		if (cards.size() > 0) {
			topCard = cards.get(cards.size()-1);
			cards.remove(cards.size()-1);
		}
		return topCard;
	}
	public Card getTopCard() {
		Card topCard = null;
		if (cards != null && cards.size() > 0) {
			topCard = cards.get(cards.size()-1);
		}
		return topCard;
	}	
	public boolean addCard(Card aCard, boolean validate) {
		if (cards == null) {
			cards = new ArrayList<Card>();
		}
		if (cards.size() == 0) {
			// go ahead and add
			aCard.setScreenPosition(new Point(topLeftX, topLeftY));
			cards.add(aCard);
			return true;
		} else {
			// check last most card and make sure this card is one less than it
			Card lastCard = cards.get(cards.size()-1);
			if (!validate) {
				aCard.setScreenPosition(new Point(topLeftX,topLeftY+(cards.size()*CARD_GAP_Y)));
				cards.add(aCard);
				return true;
			} else {
				if (lastCard.getFace().ordinal()+1 == (aCard.getFace().ordinal()) &&
						CardDeck.cardColor[lastCard.getSuit().ordinal()] != CardDeck.cardColor[aCard.getSuit().ordinal()]) {
					// add 
					aCard.setScreenPosition(new Point(topLeftX,topLeftY+(cards.size()*CARD_GAP_Y)));
					cards.add(aCard);
					return true;
				} else {
					return false;
				}
			}
		}
	}
	
	public void addCardFreely(Card aCard) {
		if (cards == null) {
			cards = new ArrayList<Card>();
		}
		// set this cards x and y location
		aCard.setScreenPosition(new Point(topLeftX, topLeftY+(cards.size()*CARD_GAP_Y)));
		cards.add(aCard);

	}	
	public void render(Graphics2D g2D) {
		if (cards.size() != 0) {
			// iterate down until all cards are displayed
			for (int i=0; i < cards.size(); i++) {
				//g2D.drawImage(cards.get(i).getImage(),null, topLeftX, topLeftY+(i*CARD_GAP_Y));
				cards.get(i).render(g2D);
			}
		} else {
			// just draw an rounded rectangle here
			g2D.setColor(Color.white);
			// draw rounded rectangle
			g2D.drawRoundRect(topLeftX, topLeftY, CardDeck.CARD_WIDTH, CardDeck.CARD_HEIGHT, CARD_ROUND_ARC, CARD_ROUND_ARC);
			
		}
	}
}
