import java.awt.Graphics2D;
import java.awt.Point;
import java.util.ArrayList;

/**
 * This class is used to track a card that is moving into the target place - manually or automatically.
 * Clients use this class when it detects:
 * 	- user is moving a card around
 *  - user or game system is automatically 
 * @author figgy
 *
 */

public class MovingCard {
	private static final int CARD_GAP_Y = 20;
	private static final int ANIMATION_JUMPS = 10;

	private	ArrayList<Card> cardList;	// holds the one or more cards that are moving
										// as a group on the screen
	private Point currentPoint;			// current drawing point on screen
	private Point mouseOffsetPoint;		// tracks the original location of the mouse point
										// with the top-left location of the cardList
										// this helps to maintain the "relative" position 
	private float xAnimationJump;		// Number of steps to move in x direction when moving back to position
	private float yAnimationJump;		// Number of steps to move in y direction when moving back to position

	private Point originalMousePoint;	// this holds the original mouse position that identified this moving
										// card set
	
	public MovingCard(ArrayList<Card> cardList, Point cardTopLeftPosition, Point mousePosition) {
		this.cardList = cardList;
		currentPoint = cardTopLeftPosition;
		mouseOffsetPoint = new Point (mousePosition.x - cardTopLeftPosition.x,
							mousePosition.y - cardTopLeftPosition.y);
	}
	
	public void clear() {
		cardList = null;
		currentPoint = null;
		mouseOffsetPoint = null;
		xAnimationJump = 0.0f;
		yAnimationJump = 0.0f;
		originalMousePoint = null;
	}
	
	// this is called by client to mark the spot where animation begins....it will be coded
	//	to return card back to spot within 10 frames
	public void markAnimationSpot() {
		if (cardList == null || cardList.size() == 0) return; // nothing to animate
		Card topCard = cardList.get(0);
		// find the number of pixels to move in x direction each frame
		xAnimationJump = Math.max(1, Math.abs(topCard.getScreenPosition().x - currentPoint.x) / ANIMATION_JUMPS);
		// adjust for direction
		xAnimationJump = (currentPoint.x < topCard.getScreenPosition().x) ? xAnimationJump : -xAnimationJump;
		// find the number of pixels to move in y direction each frame
		yAnimationJump = Math.max(1, Math.abs(topCard.getScreenPosition().y - currentPoint.y) / ANIMATION_JUMPS);
		// adjust for direction
		yAnimationJump = (currentPoint.y < topCard.getScreenPosition().y) ? yAnimationJump : -yAnimationJump;
	}
	
	/**
	 * This method returns true is card is already at ideal (original) position
	 * otherwise it returns false if it had to be adjusted to get closer to ideal position
	 * @return
	 */
	public boolean moveTowardsOriginalPosition() {
		
 		if (cardList == null || cardList.size() == 0) return true;	// nothing to do really
		Card topCard = cardList.get(0);
		if (currentPoint.x == topCard.getScreenPosition().x 
				&& currentPoint.y == topCard.getScreenPosition().y) return true;
		
		if (Math.abs(currentPoint.x - topCard.getScreenPosition().x) <= Math.abs(xAnimationJump)) {
			currentPoint.x = topCard.getScreenPosition().x;
		} else {
			currentPoint.x += xAnimationJump;
		}
		if (Math.abs(currentPoint.y - topCard.getScreenPosition().y) <= Math.abs(yAnimationJump)) {
			currentPoint.y = topCard.getScreenPosition().y;
		} else {
			currentPoint.y += yAnimationJump;
		}
		return false;	// moved
		
	}
	
	public Point getOriginalCardPoint() {
		if (cardList == null || cardList.size() == 0) return null;
		
		Card firstCard = cardList.get(0);
		return new Point (firstCard.getScreenPosition());
	}
	public ArrayList<Card> getCardList() {
		return cardList;
	}

	public void setCardList(ArrayList<Card> cardList) {
		this.cardList = cardList;
	}

	public Point getCurrentPoint() {
		return currentPoint;
	}

	public void setCurrentPoint(Point currentPoint) {
		this.currentPoint = currentPoint;
	}
	
	public Point getMouseOffsetPoint() {
		return mouseOffsetPoint;
	}

	public void setMouseOffsetPoint(Point mouseOffsetPoint) {
		this.mouseOffsetPoint = mouseOffsetPoint;
	}

	public Point getOriginalMousePoint() {
		return originalMousePoint;
	}


	public void setOriginalMousePoint(Point originalMousePoint) {
		this.originalMousePoint = originalMousePoint;
	}


	/**
	 * Used to draw on screen at current position
	 * @param g2D
	 */
	public void render(Graphics2D g2D) {
		if (cardList != null && cardList.size() > 0) {
			// iterate down until all cards are displayed
			int x = currentPoint.x;
			for (int i=0; i < cardList.size(); i++) {
				int y = currentPoint.y + (i * CARD_GAP_Y);
				g2D.drawImage(cardList.get(i).getImage(),null, x, y);
				// draw halo around first card
				if (i==0) {
					g2D.drawRoundRect(x-1, y-1, CardDeck.CARD_WIDTH+1, CardDeck.CARD_HEIGHT+1, 
							CardPlayableArea.CARD_ROUND_ARC, CardPlayableArea.CARD_ROUND_ARC);
				}
			}
		}
	}
}
